/**
 *
 */
package de.intevation.gnv.artifacts;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import de.intevation.artifactdatabase.Config;
import de.intevation.artifactdatabase.FactoryBootstrap;
import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.ArtifactFactory;
import de.intevation.gnv.artifacts.context.GNVArtifactContext;
import de.intevation.gnv.artifacts.context.GNVArtifactContextFactory;
import junit.framework.TestCase;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 *
 */
public class GNVArtifactsTestCase extends TestCase {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = null;
    
    static {
        BasicConfigurator.configure();
        log = Logger.getLogger(GNVArtifactContext.class);
    }
    
    private String configurationFile = "src/test/ressources/GNVArtifactsTestCase_Configuration.xml";
    
    private FactoryBootstrap bootstrap = null;
    /**
     * Constructor
     * @param name
     */
    public GNVArtifactsTestCase(String name) {
        super(name);
    }

    /**
     * @see junit.framework.TestCase#setUp()
     */
    protected void setUp() throws Exception {
        log.debug("GNVArtifactsTestCase.setUp");
        super.setUp();
        log.info(Config.CONFIG_PROPERTY + " ==> "+configurationFile);
        System.setProperty(Config.CONFIG_PROPERTY, configurationFile);
        log.info("Bootstrap wird initialisiert.");
        bootstrap = new FactoryBootstrap();
        bootstrap.boot();
    }
    
    public void testTimeSeriesArtifact(){
        log.debug("GNVArtifactsTestCase.testTimeSeriesArtifact");
        String artefactName = "timeSeries"; 
        ArtifactFactory artifactFactory = this.getArtifactFactory(artefactName);
        assertNotNull(artifactFactory);
        log.debug("TimeSeries-ArtifactFactory is available");
        Artifact artifact = artifactFactory.createArtifact(""+System.currentTimeMillis(),bootstrap.getContext());
        assertNotNull(artifact);
        log.debug("TimeSeries-Artifact is available");
        
        // Erster Schritt
        Document outputData = artifact.describe(bootstrap.getContext());
        this.writeDocument2Log(outputData);
        artifact.feed(this.readDocument("src/test/ressources/timeseries/timeseries_step_01_feed.xml"), bootstrap.getContext());
        outputData = artifact.advance(this.readDocument("src/test/ressources/timeseries/timeseries_step_01_advance.xml"), bootstrap.getContext());
        this.writeDocument2Log(outputData);
        
        // Zweiter Schritt
        outputData = artifact.describe(bootstrap.getContext());
        this.writeDocument2Log(outputData);
        artifact.feed(this.readDocument("src/test/ressources/timeseries/timeseries_step_02_feed.xml"), bootstrap.getContext());
        outputData = artifact.advance(this.readDocument("src/test/ressources/timeseries/timeseries_step_02_advance.xml"), bootstrap.getContext());
        this.writeDocument2Log(outputData);
        
        // Dritter Schritt
        outputData = artifact.describe(bootstrap.getContext());
        this.writeDocument2Log(outputData);
        artifact.feed(this.readDocument("src/test/ressources/timeseries/timeseries_step_03_feed.xml"), bootstrap.getContext());
        outputData = artifact.advance(this.readDocument("src/test/ressources/timeseries/timeseries_step_03_advance.xml"), bootstrap.getContext());
        this.writeDocument2Log(outputData);
        
        
    }
    private Document getDocument(){
        return this.createOutputDocument();
    }
    
    private Document createOutputDocument(){
        try {
            return DocumentBuilderFactory
                .newInstance()
                .newDocumentBuilder()
                .newDocument();
        }
        catch (ParserConfigurationException e) {
            log.error(e,e);
        }
        return null;
    }

    /**
     * @param artefactName
     */
    private ArtifactFactory getArtifactFactory(String artefactName){
        log.debug("GNVArtifactsTestCase.getArtifactFactory");
        ArtifactFactory[] artifactFactories = bootstrap.getArtifactFactories();
        for (int i = 0; i < artifactFactories.length; i++){
            if (artifactFactories[i].getName().equals(artefactName)){
                log.debug("ArtifactFactory wurde gefunden.");
                return artifactFactories[i];
            }
        }
        return null;
    }
    
    protected void writeDocument2Log(Document document){
        try {
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            DOMSource source = new DOMSource(document);
            StringWriter sw = new StringWriter();
            StreamResult result =  new StreamResult(sw);
            transformer.transform(source, result);
            log.debug(sw.getBuffer().toString());
        } catch (TransformerConfigurationException e) {
            log.error(e,e);
        } catch (TransformerFactoryConfigurationError e) {
            log.error(e,e);
        } catch (TransformerException e) {
            log.error(e,e);
        }
    }
    
    
    protected Document readDocument(String fileName){
        Document returnValue = null;
        try {
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            returnValue = docBuilder.parse (new File(fileName));
        } catch (ParserConfigurationException e) {
            log.error(e,e);
        } catch (SAXException e) {
            log.error(e,e);
        } catch (IOException e) {
            log.error(e,e);
        }
        return returnValue;
    }
    

}
